/*
 * Copyright 2012-2013 Q Ethan McCallum, ExMachinaTech.net.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


package net.exmachinatech.charcuterie.pig.eval;

import java.io.IOException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;

import net.exmachinatech.charcuterie.tools.MimeTool;

import org.apache.james.mime4j.dom.Message;
import org.apache.pig.EvalFunc;
import org.apache.pig.PigWarning;
import org.apache.pig.data.DataType;
import org.apache.pig.data.Tuple;
import org.apache.pig.data.TupleFactory;
import org.apache.pig.impl.logicalLayer.schema.Schema;
import org.apache.pig.impl.logicalLayer.schema.Schema.FieldSchema;


public class EmailExtractor
	extends EvalFunc<Tuple>
{

	/**
	 * emit dates using whatever is the local default
	 */
	public EmailExtractor(){
		_tupleFactory = TupleFactory.getInstance();
		_mimeTool = new MimeTool() ;
		_dateFormatter = null ;
	}

	/**
	 * emit dates according to the provided format.  Said format
	 * must be suitable for java.text.SimpleDateFormat, as that
	 * is what we use internally.
	 */
	public EmailExtractor( final String pattern ){
		_tupleFactory = TupleFactory.getInstance();
		_mimeTool = new MimeTool() ;
		_dateFormatter = new SimpleDateFormat( pattern ) ;
	}


	/*
	 * TODO: accept list of fields-to-return as a ctor arg
	 */


	@Override
	public Tuple exec( final Tuple input ) throws IOException {

		Tuple result = null ;

		try{

			final Message rawMessage = _mimeTool.parseMessage( (String) input.get( 0 ) ) ;
			final Date rawDate = _mimeTool.getDate( rawMessage ) ;

			String formattedDate = null ;

			if( null == _dateFormatter ){
				formattedDate = rawDate.toString() ;
			}else{
				formattedDate = _dateFormatter.format( rawDate ) ;
			}

			result = _tupleFactory.newTuple( 6 ) ;

			result.set( 0 , _mimeTool.getTo( rawMessage ) ) ;
			result.set( 1 , _mimeTool.getFrom( rawMessage ) ) ;
			result.set( 2 , _mimeTool.getCC( rawMessage ) ) ;
			result.set( 3 , formattedDate ) ;
			result.set( 4 , _mimeTool.getSubject( rawMessage ) ) ;
			result.set( 5 , _mimeTool.getBodyText( rawMessage ) ) ;

		}catch( final Exception swallowed ){
			warn( swallowed.getMessage() , PigWarning.UDF_WARNING_1 ) ;
			throw( new IOException( swallowed ) ) ;
		}

		return( result ) ;

	}

	// borrows heavily from examples in Pig UDF docs
	public Schema outputSchema( final Schema input ){

		 try{

			final int fieldCount = input.size() ;

			if( fieldCount != 1 ){
				throw( new RuntimeException( "expected a single input parameter" ) ) ;
			}


			// first, check that all the input parameters are of the proper type
			final byte inputFieldSchemaType = input.getField(0).type ;
			if( DataType.CHARARRAY != inputFieldSchemaType ){
				final String message = "Expected character data type; instead got " + DataType.findTypeName( inputFieldSchemaType ) ;
				throw( new RuntimeException( message )) ;
			}


			Schema tupleSchema = new Schema();

			tupleSchema.add( new FieldSchema( "from" , DataType.CHARARRAY )) ;
			tupleSchema.add( new FieldSchema( "to" , DataType.CHARARRAY )) ;
			tupleSchema.add( new FieldSchema( "cc" , DataType.CHARARRAY )) ;
			tupleSchema.add( new FieldSchema( "date" , DataType.CHARARRAY )) ;
			tupleSchema.add( new FieldSchema( "subject" , DataType.CHARARRAY )) ;
			tupleSchema.add( new FieldSchema( "body" , DataType.CHARARRAY )) ;

            final Schema result = new Schema(
        		new Schema.FieldSchema(
    				"email" ,
    				tupleSchema, DataType.TUPLE
				)
    		) ;

			return( result ) ;

		 }catch( final Exception rethrow ){
			 throw( new RuntimeException( rethrow ) ) ;
		 }

	}

	private final TupleFactory _tupleFactory ;
	private final MimeTool _mimeTool ;
	private final DateFormat _dateFormatter ;

}
 // EmailExtractor
