/*
 * Copyright 2012-2013 Q Ethan McCallum, ExMachinaTech.net.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


package net.exmachinatech.charcuterie.pig.eval;

import java.io.IOException;
import java.util.ArrayList;

import org.apache.pig.EvalFunc;
import org.apache.pig.PigWarning;
import org.apache.pig.data.DataType;
import org.apache.pig.data.Tuple;
import org.apache.pig.data.TupleFactory;
import org.apache.pig.impl.logicalLayer.schema.Schema;
import org.apache.pig.impl.logicalLayer.schema.Schema.FieldSchema;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.select.Elements;

/**
 * Extract HTML fields from a document.
 */

public class HTMLExtractor
	extends EvalFunc<Tuple>
{

	public HTMLExtractor(){
		_tupleFactory = TupleFactory.getInstance();
		_selectorExpressions = new ArrayList<String>() ;
		_stripHTML = false ;
	}

	public HTMLExtractor( final boolean stripHTML ){
		this() ;
		_stripHTML = stripHTML ;
	}

	public Tuple exec( final Tuple input ) throws IOException {

		final int expressionCount = input.size() - 1 ;

        Tuple result = null ;

		_selectorExpressions.clear() ;

		try{

			result = _tupleFactory.newTuple( expressionCount ) ;

			final String rawHTML = (String) input.get( 0 ) ;

			for( int ix = 1 ; ix <= expressionCount ; ++ix ){
				_selectorExpressions.add( (String) input.get( ix ) ) ;
			}


			final Document doc = Jsoup.parse( rawHTML ) ;

			// it's OK if the Jsoup Selector expression returns multiple values, or even
			// no values at all; Document.select() will return a single Element value that
			// we can convert to a String.
		    for( int ix = 0 ; ix < expressionCount ; ++ix ){

		    	final Elements found = doc.select( _selectorExpressions.get( ix ) ) ;
		    	String toAdd = null ;

		    	if( _stripHTML ){
		    		toAdd = found.text().trim() ;
		    	}else{
		    		toAdd = found.toString().trim() ;
		    	}

		    	result.set( ix , toAdd ) ;

		    }

		}catch( final Exception swallowed ){
			warn( swallowed.getMessage() , PigWarning.UDF_WARNING_1 ) ;
		}

		return( result ) ;

	}

	// borrows heavily from examples in Pig UDF docs
	public Schema outputSchema( final Schema input ){

		 try{

			 Schema tupleSchema = new Schema();

			 final int fieldCount = input.size() ;

			 if( fieldCount < 2 ){
				 throw( new RuntimeException( "expected two or more input parameters" ) ) ;
			 }


			 // first, check that all the input parameters are of the proper type
			 for( int ix = 0 ; ix < fieldCount ; ++ix ){

				 final byte inputFieldSchemaType = input.getField(ix).type ;
				 if( DataType.CHARARRAY != inputFieldSchemaType ){
					 final String message = "Expected character type in field " + ix + "; instead got " + DataType.findTypeName( inputFieldSchemaType ) ;
					 throw( new RuntimeException( message )) ;
				 }

			 }


			 // next, build the output schema.
			 // sic - notice, here, we start from input index 1, not 0:
			 // the first input parameter is the HTML to process, and
			 // the rest of the parameters mark Jsoup expressions to
			 // extract from that HTML
			 for( int ix = 1 ; ix < fieldCount ; ++ix ){
				 tupleSchema.add( new FieldSchema( null , DataType.CHARARRAY )) ;
			 }

            final Schema result = new Schema(
        		new Schema.FieldSchema(
    				getSchemaName(this.getClass().getName().toLowerCase(), input),
    				tupleSchema, DataType.TUPLE
				)
    		) ;

			return( result ) ;

		 }catch( final Exception rethrow ){
			 throw( new RuntimeException( rethrow ) ) ;
		 }

	}

	private final TupleFactory _tupleFactory ;
	private final ArrayList<String> _selectorExpressions ;
	private boolean _stripHTML ;

} // public class HTMLExtractor
